// src/services/api/room.ts
import Taro from '@tarojs/taro'
import { useUserStore } from '@/stores/user'
import { useRoomStore, type Room, type RoomUser } from '@/stores/room'

// 添加 RecentRoom 接口定义
interface RecentRoom {
  id: string;
  name: string;
  game: string;
  time: number;
  needPassword?: boolean;
}

// Mock房间数据（测试用）
const mockRooms = {
  'room_001': {
    id: 'room_001',
    name: '海龟汤房间',
    gameId: '1',
    gameTitle: '海龟汤',
    maxPlayers: 10,
    visibility: 'public',
    hosterId: 'host_123',
    hosterName: '主持人',
    createTime: Date.now() - 3600000,
    status: 'waiting',
    users: [
      {
        id: 'host_123',
        name: '主持人',
        avatar: '',
        role: 'hoster',
        joinTime: Date.now() - 3600000
      }
    ]
  },
  'room_002': {
    id: 'room_002',
    name: '狼人杀小队',
    gameId: '2',
    gameTitle: '狼人杀',
    maxPlayers: 8,
    visibility: 'private',
    password: '1234',
    hosterId: 'host_456',
    hosterName: '狼王',
    createTime: Date.now() - 86400000,
    status: 'waiting',
    users: [
      {
        id: 'host_456',
        name: '狼王',
        avatar: '',
        role: 'hoster',
        joinTime: Date.now() - 86400000
      }
    ]
  }
};

// 房间API - 简化版
export const roomAPI = {
  // 获取最近房间
  async getRecentRooms() {
    try {
      // 从缓存获取
      const cachedRooms = Taro.getStorageSync('recentRooms')
      if (cachedRooms) {
        return JSON.parse(cachedRooms)
      }
      
      // 返回默认mock数据
      return [
        { id: 'room_001', name: '海龟汤房间', game: '海龟汤', time: Date.now() - 3600000, needPassword: false },
        { id: 'room_002', name: '狼人杀小队', game: '狼人杀', time: Date.now() - 86400000, needPassword: true }
      ]
    } catch (error) {
      console.error('获取最近房间失败:', error)
      return []
    }
  },
  
  // 检查房间是否存在
  async checkRoomExist(roomId: string) {
    try {
      // 先查询本地存储
      const recentRooms = Taro.getStorageSync('recentRooms')
      if (recentRooms) {
        const rooms = JSON.parse(recentRooms)
        const room = rooms.find((r: any) => r.id === roomId)
        if (room) {
          return { exists: true, needPassword: room.needPassword }
        }
      }
      
      // 如果本地没有，检查mock数据
      const mockRoom = mockRooms[roomId]
      if (mockRoom) {
        return { 
          exists: true, 
          needPassword: mockRoom.visibility === 'private' 
        }
      }
      
      return { exists: false }
    } catch (error) {
      console.error('检查房间失败:', error)
      return { exists: false }
    }
  },
  
  // 验证房间密码
  async verifyRoomPassword(roomId: string, password: string) {
    try {
      // 简化实现：仅检查mock数据中的密码
      const mockRoom = mockRooms[roomId]
      if (mockRoom && mockRoom.visibility === 'private') {
        return { valid: mockRoom.password === password }
      }
      
      return { valid: true }
    } catch (error) {
      console.error('验证房间密码失败:', error)
      return { valid: false }
    }
  },
  
  // 创建新房间
  async createRoom(roomData: Room) {
    const userStore = useUserStore()
    
    // 检查用户是否注册
    if (!userStore.isRegistered) {
      return new Promise<{roomId: string | null, success: boolean, message?: string}>((resolve) => {
        Taro.showModal({
          title: '完善信息',
          content: '创建房间需要您的昵称和头像信息，是否现在完善？',
          confirmText: '立即完善',
          cancelText: '暂不',
          success: async (res) => {
            if (res.confirm) {
              const result = await userStore.registerUser()
              if (result) {
                // 用户注册成功，继续创建房间
                const roomResult = await this.createRoomAction(roomData)
                resolve(roomResult)
              } else {
                resolve({ roomId: null, success: false, message: '用户信息获取失败' })
              }
            } else {
              resolve({ roomId: null, success: false, message: '需要完善用户信息' })
            }
          }
        })
      })
    } else {
      // 用户已注册，直接创建房间
      return this.createRoomAction(roomData)
    }
  },
  
  // 实际创建房间的操作
  async createRoomAction(roomData: Room) {
    try {
      const roomStore = useRoomStore()
      
      // 本地存储一份房间信息
      const recentRooms = Taro.getStorageSync('recentRooms') || '[]'
      const rooms = JSON.parse(recentRooms)
      
      // 添加到最近房间
      const roomInfo = {
        id: roomData.id,
        name: roomData.name,
        game: roomData.gameTitle,
        time: roomData.createTime,
        needPassword: roomData.visibility === 'private'
      }
      
      rooms.unshift(roomInfo)
      // 只保留最近10个房间
      if (rooms.length > 10) {
        rooms.pop()
      }
      
      Taro.setStorageSync('recentRooms', JSON.stringify(rooms))
      
      // 在内存中保存当前房间
      roomStore.setCurrentRoom(roomData)
      
      // 简化实现：仅存储在本地缓存
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      roomsObj[roomData.id] = roomData
      Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      
      return { roomId: roomData.id, success: true }
    } catch (error) {
      console.error('创建房间失败:', error)
      return { roomId: null, success: false, message: '创建房间失败' }
    }
  },
  
  // 加入房间
  async joinRoom(roomId: string) {
    const userStore = useUserStore()
    
    // 检查用户是否注册
    if (!userStore.isRegistered) {
      return new Promise<{success: boolean, message?: string}>((resolve) => {
        Taro.showModal({
          title: '完善信息',
          content: '加入房间需要您的昵称和头像信息，是否现在完善？',
          confirmText: '立即完善',
          cancelText: '暂不',
          success: async (res) => {
            if (res.confirm) {
              const result = await userStore.registerUser()
              if (result) {
                // 用户注册成功，继续加入房间
                const joinResult = await this.joinRoomAction(roomId)
                resolve(joinResult)
              } else {
                resolve({ success: false, message: '用户信息获取失败' })
              }
            } else {
              resolve({ success: false, message: '需要完善用户信息' })
            }
          }
        })
      })
    } else {
      // 用户已注册，直接加入房间
      return this.joinRoomAction(roomId)
    }
  },
  
  // 实际加入房间的操作
  async joinRoomAction(roomId: string) {
    try {
      const userStore = useUserStore()
      const roomStore = useRoomStore()
      
      // 构建用户数据
      const userData: RoomUser = {
        id: userStore.openid || `temp_${Date.now()}`,
        name: userStore.nickname || '玩家',
        avatar: userStore.avatar || '',
        role: 'player',
        joinTime: Date.now()
      }
      
      // 查找房间 - 先从本地缓存查询
      let room: Room | null = null
      
      // 检查本地缓存中是否有该房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        room = roomsObj[roomId]
      } else if (mockRooms[roomId]) {
        // 使用mock数据
        room = mockRooms[roomId] as Room
      }
      
      if (!room) {
        return { success: false, message: '房间不存在' }
      }
      
      // 检查房间是否已满
      if (room.users && room.users.length >= room.maxPlayers) {
        return { success: false, message: '房间已满' }
      }
      
      // 检查用户是否已在房间中
      const existingUser = room.users.find(u => u.id === userData.id)
      if (!existingUser) {
        // 添加用户到房间
        room.users.push(userData)
        
        // 更新本地缓存
        roomsObj[roomId] = room
        Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      }
      
      // 保存到roomStore
      roomStore.setCurrentRoom(room)
      
      // 设置当前房间ID
      userStore.setCurrentRoom(roomId)
      
      // 保存到最近加入的房间
      this.saveToRecentRooms({
        id: room.id,
        name: room.name,
        game: room.gameTitle,
        time: Date.now(),
        needPassword: room.visibility === 'private'
      })
      
      return { success: true }
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  },
  
  // 保存到最近加入的房间
  async saveToRecentRooms(roomInfo: RecentRoom) {
    try {
      const recentRooms = Taro.getStorageSync('recentRooms') || '[]'
      const rooms = JSON.parse(recentRooms)
      
      // 检查是否已存在
      const existingIndex = rooms.findIndex((r: any) => r.id === roomInfo.id)
      if (existingIndex !== -1) {
        // 移除已存在的
        rooms.splice(existingIndex, 1)
      }
      
      // 添加到最前面
      rooms.unshift(roomInfo)
      
      // 只保留最近10个房间
      if (rooms.length > 10) {
        rooms.pop()
      }
      
      Taro.setStorageSync('recentRooms', JSON.stringify(rooms))
      
      // 更新roomStore的recentRooms
      const roomStore = useRoomStore()
      roomStore.loadRecentRooms(rooms)
    } catch (error) {
      console.error('保存最近房间失败:', error)
    }
  },
  
  // 退出房间
  async leaveRoom(roomId: string) {
    try {
      const userStore = useUserStore()
      const roomStore = useRoomStore()
      
      // 从本地缓存找到房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        const room = roomsObj[roomId]
        
        // 过滤掉当前用户
        room.users = room.users.filter(u => u.id !== userStore.openid)
        
        // 更新本地缓存
        roomsObj[roomId] = room
        Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      }
      
      // 清除当前房间信息
      userStore.setCurrentRoom('')
      roomStore.setCurrentRoom(null)
      
      return { success: true }
    } catch (error) {
      console.error('退出房间失败:', error)
      return { success: false, message: '退出房间失败' }
    }
  },
  
  // 获取房间信息
  async getRoomInfo(roomId: string) {
    try {
      // 检查本地缓存中是否有该房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        const roomStore = useRoomStore()
        const roomData = roomsObj[roomId]
        roomStore.setCurrentRoom(roomData)
        return roomData
      }
      
      // 如果本地没有，使用mock数据
      if (mockRooms[roomId]) {
        const roomStore = useRoomStore()
        const roomData = mockRooms[roomId]
        roomStore.setCurrentRoom(roomData as Room)
        return roomData
      }
      
      return null
    } catch (error) {
      console.error('获取房间信息失败:', error)
      return null
    }
  }
  
  // 监听房间变化功能已被移除，以简化实现
}